# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating a broker."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.events import eventflow_operations
from googlecloudsdk.command_lib.events import flags
from googlecloudsdk.command_lib.events import resource_args
from googlecloudsdk.command_lib.kuberun import connection_context
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core import log
from googlecloudsdk.core.console import progress_tracker


_DEFAULT_BROKER_NAME = 'default'

_INJECTION_LABELS = {'knative-eventing-injection': 'enabled'}


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class Create(base.Command):
  """Create a broker to initialize a namespace for eventing."""

  detailed_help = {
      'DESCRIPTION':
          """
          {description}
          Creates a new broker for the given namespace.
          This command is only available with Cloud Run for Anthos.
          """,
      'EXAMPLES':
          """
          To create a broker in the default namespace, run:

              $ {command} BROKER

          To create a broker in namespace ``NAMESPACE'', run:

              $ {command} BROKER --namespace=NAMESPACE
          """,
  }

  @staticmethod
  def Args(parser):
    flags.AddBrokerArg(parser)
    namespace_presentation = presentation_specs.ResourcePresentationSpec(
        '--namespace',
        resource_args.GetCoreNamespaceResourceSpec(),
        'Namespace to create the Broker in.',
        required=True,
        prefixes=False)
    concept_parsers.ConceptParser(
        [namespace_presentation]).AddToParser(parser)

  def Run(self, args):
    """Executes when the user runs the create command."""
    conn_context = connection_context.EventsConnectionContext(args)

    namespace_ref = args.CONCEPTS.namespace.Parse()

    with eventflow_operations.Connect(conn_context) as client:
      client.CreateBroker(namespace_ref.Name(), args.BROKER)

      broker_full_name = 'namespaces/{}/brokers/{}'.format(
          namespace_ref.Name(), args.BROKER)

      with progress_tracker.StagedProgressTracker(
          'Creating Broker...',
          [progress_tracker.Stage('Creating Broker...')]) as tracker:
        client.PollBroker(broker_full_name, tracker)

    log.status.Print('Created broker [{}] in namespace [{}].'.format(
        args.BROKER, namespace_ref.Name()))



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to show metadata for a specified folder."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.resource_manager import folders
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.resource_manager import flags


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Describe(base.DescribeCommand):
  """Show metadata for a folder.

  Shows metadata for a folder, given a valid folder ID.

  This command can fail for the following reasons:
      * The folder specified does not exist.
      * The active account does not have permission to access the given
        folder.

  ## EXAMPLES

  The following command prints metadata for a folder with the ID `3589215982`:

    $ {command} 3589215982
  """

  @staticmethod
  def Args(parser):
    flags.FolderIdArg('you want to describe.').AddToParser(parser)

  def Run(self, args):
    return folders.GetFolder(args.id)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of objects set-acl command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


class SetAcl(base.Command):
  """Set the access control list for one or more objects."""

  detailed_help = {
      'DESCRIPTION':
          """
      *{command}* allows you to set an access control list on one or more
      objects. Using *{command}* entirely replaces any existing ACL on the
      object.

      See [Predefined ACLs](https://cloud.google.com/storage/docs/access-control/lists#predefined-acl)
      for a list of canned ACLs.

      Note: to make an object publicly readable, it is recommended to use
      *{parent_command} add-acl-grant*, to avoid accidentally removing OWNER
      permissions.
      """,
      'EXAMPLES':
          """
      To simply specify one of the canned ACL names:

        $ {command} gs://bucket/file.txt --canned-acl=private

      To define fine-grained control over your data, first save the ACL to a
      file and edit the file:

        $ {parent_command} get-acl gs://bucket/object.txt > acl.txt

      To then write any modifiations to `acl.txt` as the ACL on the bucket:

        $ {command} gs://bucket/object.txt --file=acl.txt
      """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument('url', nargs='+', help='The object URL(s) to modify.')
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        '--canned-acl',
        help='The name of a predefined ACL.')
    group.add_argument(
        '--file',
        help='A file path that contains an ACL as JSON or YAML.')
    parser.add_argument(
        '-a',
        '--all-versions',
        action='store_true',
        help='Perform operation on all object versions.')

  def Run(self, args):
    del args  # Unused.
    raise NotImplementedError



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""General utilities for copies."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.storage import cloud_api
from googlecloudsdk.api_lib.storage import errors


def get_no_clobber_message(destination_url):
  """Returns standardized no clobber warning."""
  return 'Skipping existing destination item (no-clobber): {}'.format(
      destination_url)


def check_for_cloud_clobber(user_request_args, api_client,
                            destination_resource):
  """Returns if cloud destination object exists if no-clobber enabled."""
  if not (user_request_args and user_request_args.no_clobber):
    return False
  try:
    api_client.get_object_metadata(
        destination_resource.storage_url.bucket_name,
        destination_resource.storage_url.object_name,
        fields_scope=cloud_api.FieldsScope.SHORT)
  except errors.NotFoundError:
    return False
  return True


def get_generation_match_value(request_config):
  """Prioritizes user-input generation over no-clobber zero value."""
  if request_config.precondition_generation_match is not None:
    return request_config.precondition_generation_match
  if request_config.no_clobber:
    return 0
  return None



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import ABC, abstractmethod

from google.cloud.pubsublite.types.partition import Partition
from google.cloud.pubsublite_v1.types.common import PubSubMessage


class RoutingPolicy(ABC):
    """A policy for how to route messages."""

    @abstractmethod
    def route(self, message: PubSubMessage) -> Partition:
        """
        Route a message to a given partition.
        Args:
          message: The message to route

        Returns: The partition to route to

        """
        raise NotImplementedError()



from __future__ import absolute_import

from .abc import DefaultMapping


class _DefaultSize(object):

  def __getitem__(self, _):
    return 1

  def __setitem__(self, _, value):
    assert value == 1

  def pop(self, _):
    return 1


class Cache(DefaultMapping):
  """Mutable mapping to serve as a simple cache or cache base class."""

  __size = _DefaultSize()

  def __init__(self, maxsize, getsizeof=None):
    if getsizeof:
      self.getsizeof = getsizeof
    if self.getsizeof is not Cache.getsizeof:
      self.__size = dict()
    self.__data = dict()
    self.__currsize = 0
    self.__maxsize = maxsize

  def __repr__(self):
    return '%s(%r, maxsize=%r, currsize=%r)' % (
        self.__class__.__name__,
        list(self.__data.items()),
        self.__maxsize,
        self.__currsize,
    )

  def __getitem__(self, key):
    try:
      return self.__data[key]
    except KeyError:
      return self.__missing__(key)

  def __setitem__(self, key, value):
    maxsize = self.__maxsize
    size = self.getsizeof(value)
    if size > maxsize:
      raise ValueError('value too large')
    if key not in self.__data or self.__size[key] < size:
      while self.__currsize + size > maxsize:
        self.popitem()
    if key in self.__data:
      diffsize = size - self.__size[key]
    else:
      diffsize = size
    self.__data[key] = value
    self.__size[key] = size
    self.__currsize += diffsize

  def __delitem__(self, key):
    size = self.__size.pop(key)
    del self.__data[key]
    self.__currsize -= size

  def __contains__(self, key):
    return key in self.__data

  def __missing__(self, key):
    raise KeyError(key)

  def __iter__(self):
    return iter(self.__data)

  def __len__(self):
    return len(self.__data)

  @property
  def maxsize(self):
    """The maximum size of the cache."""
    return self.__maxsize

  @property
  def currsize(self):
    """The current size of the cache."""
    return self.__currsize

  @staticmethod
  def getsizeof(value):
    """Return the size of a cache element's value."""
    return 1



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from asyncio import Future, Queue, ensure_future
from typing import Callable, NamedTuple, Dict, List, Set, Optional

from google.cloud.pubsub_v1.subscriber.message import Message

from google.cloud.pubsublite.cloudpubsub.reassignment_handler import ReassignmentHandler
from google.cloud.pubsublite.cloudpubsub.internal.single_subscriber import (
    AsyncSingleSubscriber,
)
from google.cloud.pubsublite.internal.wait_ignore_cancelled import (
    wait_ignore_cancelled,
    wait_ignore_errors,
)
from google.cloud.pubsublite.internal.wire.assigner import Assigner
from google.cloud.pubsublite.internal.wire.permanent_failable import PermanentFailable
from google.cloud.pubsublite.types import Partition

PartitionSubscriberFactory = Callable[[Partition], AsyncSingleSubscriber]


class _RunningSubscriber(NamedTuple):
    subscriber: AsyncSingleSubscriber
    poller: Future


class AssigningSingleSubscriber(AsyncSingleSubscriber, PermanentFailable):
    _assigner_factory: Callable[[], Assigner]
    _subscriber_factory: PartitionSubscriberFactory
    _reassignment_handler: ReassignmentHandler

    _subscribers: Dict[Partition, _RunningSubscriber]

    # Lazily initialized to ensure they are initialized on the thread where __aenter__ is called.
    _assigner: Optional[Assigner]
    _batches: Optional["Queue[List[Message]]"]
    _assign_poller: Future

    def __init__(
        self,
        assigner_factory: Callable[[], Assigner],
        subscriber_factory: PartitionSubscriberFactory,
        reassignment_handler: ReassignmentHandler,
    ):
        """
        Accepts a factory for an Assigner instead of an Assigner because GRPC asyncio uses the current thread's event
        loop.
        """
        super().__init__()
        self._assigner_factory = assigner_factory
        self._subscriber_factory = subscriber_factory
        self._reassignment_handler = reassignment_handler
        self._assigner = None
        self._subscribers = {}
        self._batches = None

    async def read(self) -> List[Message]:
        return await self.await_unless_failed(self._batches.get())

    async def _subscribe_action(self, subscriber: AsyncSingleSubscriber):
        batch = await subscriber.read()
        await self._batches.put(batch)

    async def _start_subscriber(self, partition: Partition):
        new_subscriber = self._subscriber_factory(partition)
        await new_subscriber.__aenter__()
        poller = ensure_future(
            self.run_poller(lambda: self._subscribe_action(new_subscriber))
        )
        self._subscribers[partition] = _RunningSubscriber(new_subscriber, poller)

    async def _stop_subscriber(self, running: _RunningSubscriber):
        running.poller.cancel()
        await wait_ignore_cancelled(running.poller)
        await running.subscriber.__aexit__(None, None, None)

    async def _assign_action(self):
        assignment: Set[Partition] = await self._assigner.get_assignment()
        old_assignment: Set[Partition] = set(self._subscribers.keys())
        added_partitions = assignment - old_assignment
        removed_partitions = old_assignment - assignment
        for partition in added_partitions:
            await self._start_subscriber(partition)
        for partition in removed_partitions:
            subscriber = self._subscribers[partition]
            del self._subscribers[partition]
            await self._stop_subscriber(subscriber)
        maybe_awaitable = self._reassignment_handler.handle_reassignment(
            old_assignment, assignment
        )
        if maybe_awaitable is not None:
            await maybe_awaitable

    async def __aenter__(self):
        self._batches = Queue()
        self._assigner = self._assigner_factory()
        await self._assigner.__aenter__()
        self._assign_poller = ensure_future(self.run_poller(self._assign_action))
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        self._assign_poller.cancel()
        await wait_ignore_errors(self._assign_poller)
        await wait_ignore_errors(
            self._assigner.__aexit__(exc_type, exc_value, traceback)
        )
        for running in self._subscribers.values():
            await wait_ignore_errors(self._stop_subscriber(running))
        pass



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of objects rewrite command for transforming cloud objects."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage import flags


@base.Hidden
class Rewrite(base.Command):
  """Rewrite Cloud Storage objects."""

  detailed_help = {
      'DESCRIPTION':
          """
      Rewrite Cloud Storage objects.
      """,
      'EXAMPLES':
          """

      Rewrite a Google Cloud Storage object's with a new encryption key:

        $ {command} gs://bucket/my-object --update-encryption-key

      Change the storage class on all JPG objects in the bucket:

        $ {command} gs://bucket/*.jpg --storage-class nearline
      """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'url', nargs='+', help='Specifies URL of objects to rewrite.')
    parser.add_argument(
        '-I',
        '--stdin',
        action='store_true',
        help='Reads the list of objects to rewrite from stdin. This allows you'
        ' to run a program that generates the list of objects to rewrite.')
    parser.add_argument(
        '-k',
        '--update-encryption-key',
        action='store_true',
        help='Rewrite objects with the current gcloud storage encryption key.'
        ' The value for the encryption key may be either a base64-encoded CSEK'
        ' or a fully qualified KMS key name. If no value is specified for'
        ' the encryption key, gcloud ignores this flag. Instead, rewritten'
        " objects' are encrypted with the bucket's default KMS key, if one is"
        ' set, or Google-managed encryption, if no default KMS key is set.')
    parser.add_argument(
        '-O',
        '--do-not-preserve-acls',
        action='store_true',
        help='When a bucket has uniform bucket-level access (UBLA) enabled,'
        ' this flag is required and will skip all ACL checks. When a bucket has'
        " UBLA disabled, the -O flag rewrites objects with the bucket's default"
        ' object ACL instead of the existing object ACL. This is needed if you'
        ' do not have OWNER permission on the object.')
    parser.add_argument(
        '-R',
        '-r',
        '--recursive',
        action='store_true',
        help='Causes bucket or bucket subdirectory contents to be rewritten'
        ' recursively.')
    parser.add_argument(
        '-s',
        '--storage-class',
        action='store_true',
        help='Rewrites objects using the specified storage class.')
    flags.add_precondition_flags(parser)

  def Run(self, args):
    del args  # Unused.
    raise NotImplementedError



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""service operations wait command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.services import apikeys
from googlecloudsdk.api_lib.services import scm
from googlecloudsdk.api_lib.services import services_util
from googlecloudsdk.api_lib.services import serviceusage
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.services import common_flags

_GET_OP_FUNC_MAP = {
    'akmf': apikeys.GetOperation,
    'acf': scm.GetOperation,
}


class WaitAlpha(base.Command):
  """Waits for an operation to complete for a given operation name.

     This command will block until an operation has been marked as complete.

     ## EXAMPLES
     To wait on an operation named `operations/abc`
     to complete, run:

       $ {command} operations/abc
  """

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
    """
    common_flags.operation_flag(suffix='on which to wait').AddToParser(parser)

  def Run(self, args):
    """Run 'services operations wait'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
          with.

    Returns:
      If successful, the response from the operations.Get API call.
    """
    namespace = common_flags.get_operation_namespace(args.operation)
    get_op_func = _GET_OP_FUNC_MAP.get(namespace, serviceusage.GetOperation)

    op = services_util.WaitOperation(args.operation, get_op_func)
    services_util.PrintOperationWithResponse(op)



# coding: utf-8
"""
    Kubernetes

    No description provided (generated by Swagger Codegen
    https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: v1.14.4

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import sys
import os
import re

# python 2 and python 3 compatibility library
from six import iteritems

from ..api_client import ApiClient


class CoordinationApi(object):
  """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

  def __init__(self, api_client=None):
    if api_client is None:
      api_client = ApiClient()
    self.api_client = api_client

  def get_api_group(self, **kwargs):
    """
        get information of a group
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_api_group(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: V1APIGroup
                 If the method is called asynchronously,
                 returns the request thread.
        """
    kwargs['_return_http_data_only'] = True
    if kwargs.get('async_req'):
      return self.get_api_group_with_http_info(**kwargs)
    else:
      (data) = self.get_api_group_with_http_info(**kwargs)
      return data

  def get_api_group_with_http_info(self, **kwargs):
    """
        get information of a group
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_api_group_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: V1APIGroup
                 If the method is called asynchronously,
                 returns the request thread.
        """

    all_params = []
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')

    params = locals()
    for key, val in iteritems(params['kwargs']):
      if key not in all_params:
        raise TypeError("Got an unexpected keyword argument '%s'"
                        ' to method get_api_group' % key)
      params[key] = val
    del params['kwargs']

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params['Accept'] = self.api_client.\
        select_header_accept(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.\
        select_header_content_type(['application/json', 'application/yaml', 'application/vnd.kubernetes.protobuf'])

    # Authentication setting
    auth_settings = ['BearerToken']

    return self.api_client.call_api(
        '/apis/coordination.k8s.io/',
        'GET',
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type='V1APIGroup',
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Supplementary help for uninstalling Cloud SDK."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


class Uninstall(base.TopicCommand):
  """Supplementary help for uninstalling Google Cloud CLI.


  # Uninstalling Google Cloud CLI

  Note: For installations completed using an OS package (such as `apt-get`,
  `yum`, etc.), uninstall Google Cloud CLI via the OS package manager.

  Note: For Windows installations, execute the uninstaller.exe found under
  your Google Cloud CLI directory.

  To completely remove Google Cloud CLI, follow these instructions:

      * Locate your installation directory by running:

        $ gcloud info --format="value(installation.sdk_root)"

      * Locate your user config directory (typically `~/.config/gcloud`
        on MacOS and Linux) by running:

        $ gcloud info --format="value(config.paths.global_config_dir)"

      * Delete both these directories.

      * Additionally, remove lines sourcing `completion.bash.inc` and
        `paths.bash.inc` in your `.bashrc` or equivalent shell init file,
        if you added them during installation.

      * Review the contents of the `.boto` file in your home directory and
        remove the sections '[GoogleCompute]' and '[GSUtil]'. In addition,
        review the sections '[OAuth2]' and '[Credentials]' for settings that
        are no longer needed.

      * Some systems may have `Cache` directories such as `~/Library/Caches/` on
        Mac OS X. Find and delete these directories for your system:

        $ find ~/Library/Caches/ -type d -name "google-cloud-sdk" | xargs rm -r
  """



# Lint as: python3
# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Describe a reusable config."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.privateca import base as privateca_base
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.privateca import resource_args
from googlecloudsdk.command_lib.util.concepts import concept_parsers


class Describe(base.DescribeCommand):
  """Show details about a reusable config."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
        To show details about a reusable config, run:

          $ {command} leaf-server-tls
        """,
  }

  @staticmethod
  def Args(parser):
    concept_parsers.ConceptParser.ForResource(
        'REUSABLE_CONFIG',
        resource_args.CreateReusableConfigResourceSpec(
            location_fallthroughs=[
                resource_args.LOCATION_PROPERTY_FALLTHROUGH]),
        'The reusable config to describe.',
        required=True).AddToParser(parser)

  def Run(self, args):
    """Runs the command."""

    reusable_config = args.CONCEPTS.reusable_config.Parse()

    client = privateca_base.GetClientInstance()
    messages = privateca_base.GetMessagesModule()

    return client.projects_locations_reusableConfigs.Get(
        messages.PrivatecaProjectsLocationsReusableConfigsGetRequest(
            name=reusable_config.RelativeName()))



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to update an Anthos cluster on AWS."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.aws import clusters
from googlecloudsdk.command_lib.container.aws import flags as aws_flags
from googlecloudsdk.command_lib.container.aws import resource_args
from googlecloudsdk.command_lib.container.gkemulticloud import constants
from googlecloudsdk.command_lib.container.gkemulticloud import endpoint_util
from googlecloudsdk.command_lib.container.gkemulticloud import flags
from googlecloudsdk.command_lib.container.gkemulticloud import operations
from googlecloudsdk.core import log

# Command needs to be in one line for the docgen tool to format properly.
_EXAMPLES = """
To update a cluster named ``my-cluster'' managed in location ``us-west1'', run:

$ {command} my-cluster --location=us-west1 --cluster-version=CLUSTER_VERSION
"""


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class Update(base.UpdateCommand):
  """Update an Anthos cluster on AWS."""

  detailed_help = {'EXAMPLES': _EXAMPLES}

  @staticmethod
  def Args(parser):
    resource_args.AddAwsClusterResourceArg(parser, 'to update')
    flags.AddClusterVersion(parser, required=False)
    flags.AddValidateOnly(parser, 'update of the cluster')
    flags.AddAdminUsers(parser, create=False)
    flags.AddRootVolumeSize(parser)
    aws_flags.AddInstanceType(parser)
    aws_flags.AddRoleArn(parser, required=False)
    aws_flags.AddRoleSessionName(parser)
    aws_flags.AddConfigEncryptionKmsKeyArn(parser, required=False)
    aws_flags.AddSecurityGroupFlagsForUpdate(parser, 'control plane replicas')
    aws_flags.AddProxyConfigForUpdate(parser)
    aws_flags.AddRootVolumeKmsKeyArn(parser)
    aws_flags.AddRootVolumeType(parser)
    aws_flags.AddRootVolumeIops(parser)

    base.ASYNC_FLAG.AddToParser(parser)
    parser.display_info.AddFormat(clusters.CLUSTERS_FORMAT)

  def Run(self, args):
    """Runs the update command."""
    with endpoint_util.GkemulticloudEndpointOverride(
        resource_args.ParseAwsClusterResourceArg(args).locationsId,
        self.ReleaseTrack()):

      cluster_ref = resource_args.ParseAwsClusterResourceArg(args)
      cluster_client = clusters.Client(track=self.ReleaseTrack())
      args.root_volume_size = flags.GetRootVolumeSize(args)
      args.root_volume_type = aws_flags.GetRootVolumeType(args)
      op = cluster_client.Update(cluster_ref, args)

      validate_only = getattr(args, 'validate_only', False)
      if validate_only:
        args.format = 'disable'
        return

      op_ref = resource_args.GetOperationResource(op)
      log.CreatedResource(op_ref, kind=constants.LRO_KIND)

      async_ = getattr(args, 'async_', False)
      if not async_:
        op_client = operations.Client(track=self.ReleaseTrack())
        op_client.Wait(op_ref,
                       'Updating cluster {}.'.format(cluster_ref.awsClustersId))

      log.UpdatedResource(
          cluster_ref, kind=constants.AWS_CLUSTER_KIND, is_async=async_)
      return cluster_client.Get(cluster_ref)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List supported locations for the Private CA API."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.privateca import base as privateca_base
from googlecloudsdk.api_lib.privateca import locations
from googlecloudsdk.calliope import base


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class ListBeta(base.ListCommand):
  """List supported locations for the Private CA preview API."""

  detailed_help = {
      'DESCRIPTION':
          """\
          Returns supported locations where resources can be managed through
          the Private CA preview API.""",
      'EXAMPLES':
          """\
          To list the locations available for the Private CA preview API:

          $ {command}

          """,
  }

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat('table(locationId:label=LOCATION_ID)')

  def Run(self, args):
    """Runs the command."""
    messages = privateca_base.GetMessagesModule('v1beta1')
    return [
        messages.Location(locationId=location)
        for location in locations.GetSupportedLocations('v1beta1')
    ]


@base.ReleaseTracks(base.ReleaseTrack.GA)
class List(base.ListCommand):
  """List supported locations for the Private CA GA API."""

  detailed_help = {
      'DESCRIPTION':
          """\
          Returns supported locations where resources can be managed through
          the Private CA GA API.""",
      'EXAMPLES':
          """\
          To list the locations available for the Private CA GA API:

          $ {command}

          """,
  }

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat('table(locationId:label=LOCATION_ID)')

  def Run(self, args):
    """Runs the command."""
    messages = privateca_base.GetMessagesModule('v1')
    return [
        messages.Location(locationId=location)
        for location in locations.GetSupportedLocations('v1')
    ]



# -*- coding: utf-8 -*- #
# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to authorize a P4SA service account to another resource."""


from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.cloudresourcemanager import projects_api
from googlecloudsdk.api_lib.dataplex import lake
from googlecloudsdk.api_lib.storage import storage_api
from googlecloudsdk.api_lib.storage import storage_util
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import exceptions as gcloud_exception
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataplex import resource_args
from googlecloudsdk.command_lib.projects import util as project_util


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class Authorize(base.Command):
  """Authorize a project service account to manage given resource.

  IAM Bindings for the service account of the primary project will be added to
  a secondary project, a storage bucket, or BigQuery dataset.
  """

  detailed_help = {
      'EXAMPLES':
          """\
          To authorize the service account in project `test-project` to
          manage another project `test-project2`, run:

            $ {command} --project=test-project --project-resource=test-project2

          To authorize the service account in project `test-project` to
          manage the storage bucket `dataplex-storage-bucket`, run:

            $ {command} --project=test-project --storage-bucket-resource=dataplex-storage-bucket

          To authorize the service account in project `test-project` to
          manage the BigQuery dataset `test-dataset` in project `test-project2`,
          run:

            $ {command} --project=test-project --bigquery-dataset-resource=test-dataset --secondary-project=test-project2
          """,
  }

  @staticmethod
  def Args(parser):
    resource_args.AddProjectArg(parser,
                                'to add service agent IAM policy binding to.')
    data_group = parser.add_group(
        mutex=True, required=True, help='Container or Object to bind P4SA.')
    data_group.add_argument(
        '--storage-bucket-resource',
        help="""The identifier of the Cloud Storage bucket to authorize the
                project on.""")
    data_group.add_argument(
        '--project-resource',
        help='The identifier of the project to authorize.')
    dataset_group = data_group.add_group(
        help='Fields to help identify the BigQuery Dataset.')
    dataset_group.add_argument(
        '--bigquery-dataset-resource',
        required=True,
        help='The identifier of the BigQuery dataset to authorize.'
    )
    dataset_group.add_argument(
        '--secondary-project',
        required=True,
        help='The identifier of the Project where BigQuery dataset resides.')

  @gcloud_exception.CatchHTTPErrorRaiseHTTPException(
      'Status code: {status_code}. {status_message}.')
  def Run(self, args):
    project_ref = args.CONCEPTS.project.Parse()
    service_account = 'service-' + str(
        project_util.GetProjectNumber(project_ref.projectsId)
    ) + '@gcp-sa-dataplex.iam.gserviceaccount.com'

    if args.IsSpecified('storage_bucket_resource'):
      return storage_api.StorageClient().AddIamPolicyBinding(
          storage_util.BucketReference(args.storage_bucket_resource),
          'serviceAccount:' + service_account, 'roles/dataplex.serviceAgent')

    if args.IsSpecified('bigquery_dataset_resource'):
      get_dataset_request = apis.GetMessagesModule(
          'bigquery', 'v2').BigqueryDatasetsGetRequest(
              datasetId=args.bigquery_dataset_resource,
              projectId=args.secondary_project)
      dataset = apis.GetClientInstance(
          'bigquery', 'v2').datasets.Get(request=get_dataset_request)
      lake.AddServiceAccountToDatasetPolicy(
          apis.GetMessagesModule('bigquery', 'v2').Dataset.AccessValueListEntry,
          dataset, service_account, 'roles/dataplex.serviceAgent')
      return apis.GetClientInstance('bigquery', 'v2').datasets.Patch(
          apis.GetMessagesModule('bigquery', 'v2').BigqueryDatasetsPatchRequest(
              datasetId=args.bigquery_dataset_resource,
              projectId=args.secondary_project,
              dataset=dataset))

    if args.IsSpecified('project_resource'):
      return projects_api.AddIamPolicyBinding(
          project_util.ParseProject(args.project_resource),
          'serviceAccount:' + service_account, 'roles/dataplex.serviceAgent')



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""--flags-file=YAML_FILE supplementary help."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


# NOTE: If the name of this topic is modified, please make sure to update all
# references to it in error messages and other help messages as there are no
# tests to catch such changes.
class CommandConventions(base.TopicCommand):
  r"""--flags-file=YAML_FILE supplementary help.

  The *--flags-file*=_YAML-FILE_ flag, available to all *gcloud* commands,
  supports complex flag values in any command interpreter.

  Complex flag values that contain command interpreter special characters may
  be difficult to specify on the command line. The combined list of _special_
  characters across commonly used command interpreters (shell, cmd.exe,
  PowerShell) is surprisingly large. Among them are ```", ', `, *, ?, [, ],
  (, ), $, %, #, ^, &, |, {, }, ;, \, <, >,``` _space_, _tab_, _newline_.
  Add to that the separator characters for *list* and *dict* valued flags,
  and it becomes all but impossible to construct portable command lines.

  The *--flags-file*=_YAML-FILE_ flag solves this problem by allowing
  command line flags to be specified in a YAML/JSON file. String, numeric, list
  and dict flag values are specified using YAML/JSON notation and quoting rules.

  Flag specification uses dictionary notation. Use a list of dictionaries for
  flags that must be specified multiple times.

  For example, this YAML file defines values for Boolean, integer, floating
  point, string, dictionary and list valued flags:

  ```
      --boolean:
      --integer: 123
      --float: 456.789
      --string: A string value.
      --dictionary:
        a=b: c,d
        e,f: g=h
        i: none
        j=k=l: m=$n,o=%p
        "y:": ":z"
        meta:
        - key: foo
          value: bar
        - key: abc
          value: xyz
      --list:
        - a,b,c
        - x,y,z
  ```

  If the file is named *my-flags.yaml* then the command line flag
  *--flags-file=my-flags.yaml* will set the specified flags on any system
  using any command interpreter. *--flags-file* may be specified in a YAML
  file, and its value can be a YAML list to reference multiple files.

  This example specifies the *--metadata* flag multiple times:

  ```
      - --metadata: abc
        --integer: 123
      - --metadata: xyz
  ```

  Each *--flags-file* arg is replaced by its contents, so normal flag
  precedence applies. For example, given *flags-1.yaml*:

  ```
      --zone: us-east2-a
  ```

  *flags-2.yaml*:

  ```
      --verbosity: info
      --zone: us-central1-a
  ```

  and command line:

  ```
      gcloud compute instances describe \
          --flags-file=flags-1.yaml my-instance --flags-file=flags-2.yaml
  ```

  the effective command line is:

  ```
      gcloud compute instances describe \
          --zone=us-east2-a my-instance --verbosity=info --zone=us-central1-a
  ```

  using zone *us-central1-a* (not *us-east2-a*, because `flags-2.yaml`, to the
  right of `flags-1.yaml`, has higher precedence).
  """



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
from typing import Optional, List

import logging

from overrides import overrides

from google.cloud.pubsublite.internal.wait_ignore_cancelled import wait_ignore_errors
from google.cloud.pubsublite.internal.wire.committer import Committer
from google.cloud.pubsublite.internal.wire.retrying_connection import (
    RetryingConnection,
    ConnectionFactory,
)
from google.api_core.exceptions import FailedPrecondition, GoogleAPICallError
from google.cloud.pubsublite.internal.wire.connection_reinitializer import (
    ConnectionReinitializer,
)
from google.cloud.pubsublite.internal.wire.connection import Connection
from google.cloud.pubsublite_v1 import Cursor
from google.cloud.pubsublite_v1.types import (
    StreamingCommitCursorRequest,
    StreamingCommitCursorResponse,
    InitialCommitCursorRequest,
)


_LOGGER = logging.getLogger(__name__)


class CommitterImpl(
    Committer,
    ConnectionReinitializer[
        StreamingCommitCursorRequest, StreamingCommitCursorResponse
    ],
):
    _initial: InitialCommitCursorRequest
    _flush_seconds: float
    _connection: RetryingConnection[
        StreamingCommitCursorRequest, StreamingCommitCursorResponse
    ]

    _next_to_commit: Optional[Cursor]
    _outstanding_commits: List[Cursor]

    _receiver: Optional[asyncio.Future]
    _flusher: Optional[asyncio.Future]
    _empty: asyncio.Event

    def __init__(
        self,
        initial: InitialCommitCursorRequest,
        flush_seconds: float,
        factory: ConnectionFactory[
            StreamingCommitCursorRequest, StreamingCommitCursorResponse
        ],
    ):
        self._initial = initial
        self._flush_seconds = flush_seconds
        self._connection = RetryingConnection(factory, self)
        self._next_to_commit = None
        self._outstanding_commits = []
        self._receiver = None
        self._flusher = None
        self._empty = asyncio.Event()
        self._empty.set()

    async def __aenter__(self):
        await self._connection.__aenter__()
        return self

    def _start_loopers(self):
        assert self._receiver is None
        assert self._flusher is None
        self._receiver = asyncio.ensure_future(self._receive_loop())
        self._flusher = asyncio.ensure_future(self._flush_loop())

    async def _stop_loopers(self):
        if self._receiver:
            self._receiver.cancel()
            await wait_ignore_errors(self._receiver)
            self._receiver = None
        if self._flusher:
            self._flusher.cancel()
            await wait_ignore_errors(self._flusher)
            self._flusher = None

    def _handle_response(self, response: StreamingCommitCursorResponse):
        if "commit" not in response:
            self._connection.fail(
                FailedPrecondition(
                    "Received an invalid subsequent response on the commit stream."
                )
            )
        if response.commit.acknowledged_commits > len(self._outstanding_commits):
            self._connection.fail(
                FailedPrecondition(
                    "Received a commit response on the stream with no outstanding commits."
                )
            )
        for _ in range(response.commit.acknowledged_commits):
            self._outstanding_commits.pop(0)
        if len(self._outstanding_commits) == 0:
            self._empty.set()

    async def _receive_loop(self):
        while True:
            response = await self._connection.read()
            self._handle_response(response)

    async def _flush_loop(self):
        while True:
            await asyncio.sleep(self._flush_seconds)
            await self._flush()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self._stop_loopers()
        if not self._connection.error():
            await self._flush()
        await self._connection.__aexit__(exc_type, exc_val, exc_tb)

    async def _flush(self):
        if self._next_to_commit is None:
            return
        req = StreamingCommitCursorRequest()
        req.commit.cursor = self._next_to_commit
        self._outstanding_commits.append(self._next_to_commit)
        self._next_to_commit = None
        self._empty.clear()
        try:
            await self._connection.write(req)
        except GoogleAPICallError as e:
            _LOGGER.debug(f"Failed commit on stream: {e}")

    async def wait_until_empty(self):
        await self._flush()
        await self._connection.await_unless_failed(self._empty.wait())

    def commit(self, cursor: Cursor) -> None:
        if self._connection.error():
            raise self._connection.error()
        self._next_to_commit = cursor

    @overrides
    async def stop_processing(self, error: GoogleAPICallError):
        await self._stop_loopers()

    @overrides
    async def reinitialize(
        self,
        connection: Connection[
            StreamingCommitCursorRequest, StreamingCommitCursorResponse
        ],
    ):
        await connection.write(StreamingCommitCursorRequest(initial=self._initial))
        response = await connection.read()
        if "initial" not in response:
            self._connection.fail(
                FailedPrecondition(
                    "Received an invalid initial response on the publish stream."
                )
            )
        if self._next_to_commit is None:
            if self._outstanding_commits:
                self._next_to_commit = self._outstanding_commits[-1]
        self._outstanding_commits = []
        self._start_loopers()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import (
    Union,
    AsyncIterator,
    Awaitable,
    Callable,
    Optional,
    Set,
)

from google.cloud.pubsub_v1.subscriber.message import Message

from google.cloud.pubsublite.cloudpubsub.internal.single_subscriber import (
    AsyncSubscriberFactory,
    AsyncSingleSubscriber,
)
from google.cloud.pubsublite.cloudpubsub.subscriber_client_interface import (
    AsyncSubscriberClientInterface,
)
from google.cloud.pubsublite.types import (
    SubscriptionPath,
    FlowControlSettings,
    Partition,
)
from overrides import overrides


async def _iterate_subscriber(
    subscriber: AsyncSingleSubscriber, on_failure: Callable[[], Awaitable[None]]
) -> AsyncIterator[Message]:
    try:
        while True:
            batch = await subscriber.read()
            for message in batch:
                yield message
    except:  # noqa: E722
        await on_failure()
        raise


class MultiplexedAsyncSubscriberClient(AsyncSubscriberClientInterface):
    _underlying_factory: AsyncSubscriberFactory
    _live_clients: Set[AsyncSingleSubscriber]

    def __init__(self, underlying_factory: AsyncSubscriberFactory):
        self._underlying_factory = underlying_factory
        self._live_clients = set()

    @overrides
    async def subscribe(
        self,
        subscription: Union[SubscriptionPath, str],
        per_partition_flow_control_settings: FlowControlSettings,
        fixed_partitions: Optional[Set[Partition]] = None,
    ) -> AsyncIterator[Message]:
        if isinstance(subscription, str):
            subscription = SubscriptionPath.parse(subscription)

        subscriber = self._underlying_factory(
            subscription, fixed_partitions, per_partition_flow_control_settings
        )
        await subscriber.__aenter__()
        self._live_clients.add(subscriber)

        return _iterate_subscriber(
            subscriber, lambda: self._try_remove_client(subscriber)
        )

    @overrides
    async def __aenter__(self):
        return self

    async def _try_remove_client(self, client: AsyncSingleSubscriber):
        if client in self._live_clients:
            self._live_clients.remove(client)
            await client.__aexit__(None, None, None)

    @overrides
    async def __aexit__(self, exc_type, exc_value, traceback):
        live_clients = self._live_clients
        self._live_clients = set()
        for client in live_clients:
            await client.__aexit__(None, None, None)



# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
x=sys.modules['containerregistry.tools']
  

from containerregistry.tools import patched_
setattr(x, 'patched', patched_)


from containerregistry.tools import platform_args_
setattr(x, 'platform_args', platform_args_)


from containerregistry.tools import logging_setup_
setattr(x, 'logging_setup', logging_setup_)


from containerregistry.tools import docker_appender_
setattr(x, 'docker_appender', docker_appender_)


from containerregistry.tools import docker_puller_
setattr(x, 'docker_puller', docker_puller_)


from containerregistry.tools import docker_pusher_
setattr(x, 'docker_pusher', docker_pusher_)


from containerregistry.tools import fast_puller_
setattr(x, 'fast_puller', fast_puller_)


from containerregistry.tools import fast_flatten_
setattr(x, 'fast_flatten', fast_flatten_)


from containerregistry.tools import fast_importer_
setattr(x, 'fast_importer', fast_importer_)


from containerregistry.tools import fast_pusher_
setattr(x, 'fast_pusher', fast_pusher_)


from containerregistry.tools import image_digester_
setattr(x, 'image_digester', image_digester_)





# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Used for validating parameters provided to create and update integrations."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.run.integrations import types_utils
from googlecloudsdk.command_lib.run import exceptions


def GetIntegrationValidator(integration_type, parameters):
  """Gets the integration validator based on the integration type."""
  integration = types_utils.GetIntegration(integration_type)

  if integration is None:
    raise ValueError(
        'Integration type: [{}] has not been defined in types_utils'
        .format(integration_type))

  return Validator(integration, parameters)


class Validator:
  """Validates parameters for creating and updating integrations."""

  def __init__(self, integration, parameters):
    self.integration = integration
    self.user_provided_params = parameters

  def ValidateCreateParameters(self):
    """Validates parameters provided for creating an integration.

    Three things are done for all integrations created:
      1. Check that parameters passed in are valid (exist in types_utils
        mapping) and are not misspelled. These are parameters that will
        be recognized by the control plane.
      2. Check that all required parameters are provided.
      3. Check that default values are set for parameters
        that are not provided.

    Note that user provided params may be modified in place
    if default values are missing.
    """
    self._ValidateProvidedParams()
    self._ValidateRequiredParams()
    self._SetupDefaultParams()

  def ValidateUpdateParameters(self):
    """Checks that certain parameters have not been updated.

    This firstly checks that the parameters provided exist in the mapping
    and thus are recognized the control plane.
    """
    self._ValidateProvidedParams()
    self._CheckForInvalidUpdateParameters()

  def _CheckForInvalidUpdateParameters(self):
    """Raises an exception that lists the parameters that can't be changed."""
    invalid_params = []
    for param_name, param in self.integration['parameters'].items():
      update_allowed = param.get('update_allowed', True)
      if not update_allowed and param_name in self.user_provided_params:
        invalid_params.append(param_name)

    if invalid_params:
      raise exceptions.ArgumentError(
          ('The following parameters: {} cannot be changed once the ' +
           'integration has been created')
          .format(self._RemoveEncoding(invalid_params))
      )

  def _ValidateProvidedParams(self):
    """Checks that the user provided parameters exist in the mapping."""
    invalid_params = []
    for param in self.user_provided_params:
      if param not in self.integration['parameters']:
        invalid_params.append(param)

    if invalid_params:
      raise exceptions.ArgumentError(
          'The following parameters: {} are not allowed'.format(
              self._RemoveEncoding(invalid_params))
      )

  def _ValidateRequiredParams(self):
    """Checks that required parameters are provided by the user."""
    missing_required_params = []
    for param_name, param in self.integration['parameters'].items():
      required = param.get('required', False)

      if required and param_name not in self.user_provided_params:
        missing_required_params.append(param_name)

    if missing_required_params:
      raise exceptions.ArgumentError(
          ('The following parameters: {} are required to create an ' +
           'integration of type [{}]')
          .format(
              self._RemoveEncoding(missing_required_params),
              self.integration['name'])
      )

  def _RemoveEncoding(self, elements):
    """Removes encoding for each element in the list.

    This causes inconsistencies in the scenario test when the output
    looks like [u'domain'] instead of ['domain']

    Args:
      elements: list

    Returns:
      list[str], encoding removed from each element.
    """
    return [str(x) for x in elements]

  def _SetupDefaultParams(self):
    """Ensures that default parameters have a value if not set."""
    for param_name, param in self.integration['parameters'].items():
      if ('default' in param and
          param_name not in self.user_provided_params):
        self.user_provided_params[param_name] = param['default']



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of objects update command for updating object settings."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage import flags


@base.Hidden
class Update(base.Command):
  """Update Cloud Storage objects."""

  detailed_help = {
      'DESCRIPTION':
          """
      Update Cloud Storage objects.
      """,
      'EXAMPLES':
          """

      Update a Google Cloud Storage object's custom-metadata:

        $ *{command}* gs://bucket/my-object --custom-metadata=key1=value1,key2=value2

      Set a temporary hold on JPG images:

        $ *{command}* gs://bucket/*.jpg --temporary-hold

       You can also provide a precondition on an object's metageneration in
       order to avoid potential race conditions:

        $ *{command}* gs://bucket/*.jpg --temporary-hold --if-metageneration-match=123456789
      """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'url', nargs='+', help='Specifies URL of objects to update.')
    parser.add_argument(
        '--event-based-hold',
        action=arg_parsers.StoreTrueFalseAction,
        help='Enables or disables an event-based hold on objects.')
    parser.add_argument(
        '--temporary-hold',
        action=arg_parsers.StoreTrueFalseAction,
        help='Enables or disables a temporary hold on objects.')
    flags.add_precondition_flags(parser)
    flags.add_object_metadata_flags(parser)

  def Run(self, args):
    del args  # Unused.
    raise NotImplementedError



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils for GKE Hub Policy Controller commands."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.container.fleet.features import base
from googlecloudsdk.core import exceptions
from googlecloudsdk.core.console import console_io


def select_memberships(args):
  """Returns a list of memberships to which to apply the command, given the arguments.

  Args:
    args: object containing arguments passed as flags with the command

  Returns:
    memberships: A list of membership name strings
  """
  memberships = []
  all_memberships = base.ListMemberships()

  if args.all_memberships:
    memberships = all_memberships
  elif args.memberships:
    memberships = args.memberships.split(',')
    for membership in memberships:
      if membership not in all_memberships:
        raise exceptions.Error('Membership {} not found'.format(membership))
  else:
    index = console_io.PromptChoice(
        options=all_memberships, message='Please specify a membership:\n')
    memberships = [all_memberships[index]]

  if not memberships:
    raise exceptions.Error('A membership is required for this command.')

  return memberships


def set_poco_hub_config_parameters_from_args(args, messages):
  """Returns a Policy Controller Hub Config object with parameters as passed in the command flags.

  Args:
    args: object containing arguments passed as flags with the command
    messages: GKE Hub proto messages

  Returns:
    poco_hub_config: Policy Controller Hub Config object with parameters filled
    out
  """
  poco_hub_config = messages.PolicyControllerHubConfig(
    )
  if args.audit_interval_seconds:
    poco_hub_config.auditIntervalSeconds = args.audit_interval_seconds
  if args.exemptable_namespaces:
    exemptable_namespaces = args.exemptable_namespaces.split(',')
    poco_hub_config.exemptableNamespaces = exemptable_namespaces
  if args.log_denies_enabled is not None:
    poco_hub_config.logDeniesEnabled = args.log_denies_enabled
  if args.referential_rules_enabled is not None:
    poco_hub_config.referentialRulesEnabled = args.referential_rules_enabled
  if args.template_library_installed is not None:
    poco_hub_config.templateLibraryConfig = messages.PolicyControllerTemplateLibraryConfig(
        included=args.template_library_installed)
  return poco_hub_config



# -*- coding: utf-8 -*-

# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from google.cloud.pubsublite_v1.services.admin_service.async_client import (
    AdminServiceAsyncClient,
)
from google.cloud.pubsublite_v1.services.admin_service.client import AdminServiceClient
from google.cloud.pubsublite_v1.services.cursor_service.async_client import (
    CursorServiceAsyncClient,
)
from google.cloud.pubsublite_v1.services.cursor_service.client import (
    CursorServiceClient,
)
from google.cloud.pubsublite_v1.services.partition_assignment_service.async_client import (
    PartitionAssignmentServiceAsyncClient,
)
from google.cloud.pubsublite_v1.services.partition_assignment_service.client import (
    PartitionAssignmentServiceClient,
)
from google.cloud.pubsublite_v1.services.publisher_service.async_client import (
    PublisherServiceAsyncClient,
)
from google.cloud.pubsublite_v1.services.publisher_service.client import (
    PublisherServiceClient,
)
from google.cloud.pubsublite_v1.services.subscriber_service.async_client import (
    SubscriberServiceAsyncClient,
)
from google.cloud.pubsublite_v1.services.subscriber_service.client import (
    SubscriberServiceClient,
)
from google.cloud.pubsublite_v1.services.topic_stats_service.async_client import (
    TopicStatsServiceAsyncClient,
)
from google.cloud.pubsublite_v1.services.topic_stats_service.client import (
    TopicStatsServiceClient,
)
from google.cloud.pubsublite_v1.types.admin import CreateSubscriptionRequest
from google.cloud.pubsublite_v1.types.admin import CreateTopicRequest
from google.cloud.pubsublite_v1.types.admin import DeleteSubscriptionRequest
from google.cloud.pubsublite_v1.types.admin import DeleteTopicRequest
from google.cloud.pubsublite_v1.types.admin import GetSubscriptionRequest
from google.cloud.pubsublite_v1.types.admin import GetTopicPartitionsRequest
from google.cloud.pubsublite_v1.types.admin import GetTopicRequest
from google.cloud.pubsublite_v1.types.admin import ListSubscriptionsRequest
from google.cloud.pubsublite_v1.types.admin import ListSubscriptionsResponse
from google.cloud.pubsublite_v1.types.admin import ListTopicSubscriptionsRequest
from google.cloud.pubsublite_v1.types.admin import ListTopicSubscriptionsResponse
from google.cloud.pubsublite_v1.types.admin import ListTopicsRequest
from google.cloud.pubsublite_v1.types.admin import ListTopicsResponse
from google.cloud.pubsublite_v1.types.admin import OperationMetadata
from google.cloud.pubsublite_v1.types.admin import SeekSubscriptionRequest
from google.cloud.pubsublite_v1.types.admin import SeekSubscriptionResponse
from google.cloud.pubsublite_v1.types.admin import TopicPartitions
from google.cloud.pubsublite_v1.types.admin import UpdateSubscriptionRequest
from google.cloud.pubsublite_v1.types.admin import UpdateTopicRequest
from google.cloud.pubsublite_v1.types.common import AttributeValues
from google.cloud.pubsublite_v1.types.common import Cursor
from google.cloud.pubsublite_v1.types.common import PubSubMessage
from google.cloud.pubsublite_v1.types.common import Reservation
from google.cloud.pubsublite_v1.types.common import SequencedMessage
from google.cloud.pubsublite_v1.types.common import Subscription
from google.cloud.pubsublite_v1.types.common import TimeTarget
from google.cloud.pubsublite_v1.types.common import Topic
from google.cloud.pubsublite_v1.types.cursor import CommitCursorRequest
from google.cloud.pubsublite_v1.types.cursor import CommitCursorResponse
from google.cloud.pubsublite_v1.types.cursor import InitialCommitCursorRequest
from google.cloud.pubsublite_v1.types.cursor import InitialCommitCursorResponse
from google.cloud.pubsublite_v1.types.cursor import ListPartitionCursorsRequest
from google.cloud.pubsublite_v1.types.cursor import ListPartitionCursorsResponse
from google.cloud.pubsublite_v1.types.cursor import PartitionCursor
from google.cloud.pubsublite_v1.types.cursor import SequencedCommitCursorRequest
from google.cloud.pubsublite_v1.types.cursor import SequencedCommitCursorResponse
from google.cloud.pubsublite_v1.types.cursor import StreamingCommitCursorRequest
from google.cloud.pubsublite_v1.types.cursor import StreamingCommitCursorResponse
from google.cloud.pubsublite_v1.types.publisher import InitialPublishRequest
from google.cloud.pubsublite_v1.types.publisher import InitialPublishResponse
from google.cloud.pubsublite_v1.types.publisher import MessagePublishRequest
from google.cloud.pubsublite_v1.types.publisher import MessagePublishResponse
from google.cloud.pubsublite_v1.types.publisher import PublishRequest
from google.cloud.pubsublite_v1.types.publisher import PublishResponse
from google.cloud.pubsublite_v1.types.subscriber import FlowControlRequest
from google.cloud.pubsublite_v1.types.subscriber import (
    InitialPartitionAssignmentRequest,
)
from google.cloud.pubsublite_v1.types.subscriber import InitialSubscribeRequest
from google.cloud.pubsublite_v1.types.subscriber import InitialSubscribeResponse
from google.cloud.pubsublite_v1.types.subscriber import MessageResponse
from google.cloud.pubsublite_v1.types.subscriber import PartitionAssignment
from google.cloud.pubsublite_v1.types.subscriber import PartitionAssignmentAck
from google.cloud.pubsublite_v1.types.subscriber import PartitionAssignmentRequest
from google.cloud.pubsublite_v1.types.subscriber import SeekRequest
from google.cloud.pubsublite_v1.types.subscriber import SeekResponse
from google.cloud.pubsublite_v1.types.subscriber import SubscribeRequest
from google.cloud.pubsublite_v1.types.subscriber import SubscribeResponse
from google.cloud.pubsublite_v1.types.topic_stats import ComputeMessageStatsRequest
from google.cloud.pubsublite_v1.types.topic_stats import ComputeMessageStatsResponse
from google.cloud.pubsublite.admin_client_interface import AdminClientInterface
from google.cloud.pubsublite.admin_client import AdminClient

__all__ = (
    # Manual files
    "AdminClient",
    "AdminClientInterface",
    # Generated files
    "AdminServiceAsyncClient",
    "AdminServiceClient",
    "AttributeValues",
    "CommitCursorRequest",
    "CommitCursorResponse",
    "ComputeMessageStatsRequest",
    "ComputeMessageStatsResponse",
    "CreateSubscriptionRequest",
    "CreateTopicRequest",
    "Cursor",
    "CursorServiceAsyncClient",
    "CursorServiceClient",
    "DeleteSubscriptionRequest",
    "DeleteTopicRequest",
    "FlowControlRequest",
    "GetSubscriptionRequest",
    "GetTopicPartitionsRequest",
    "GetTopicRequest",
    "InitialCommitCursorRequest",
    "InitialCommitCursorResponse",
    "InitialPartitionAssignmentRequest",
    "InitialPublishRequest",
    "InitialPublishResponse",
    "InitialSubscribeRequest",
    "InitialSubscribeResponse",
    "ListPartitionCursorsRequest",
    "ListPartitionCursorsResponse",
    "ListSubscriptionsRequest",
    "ListSubscriptionsResponse",
    "ListTopicSubscriptionsRequest",
    "ListTopicSubscriptionsResponse",
    "ListTopicsRequest",
    "ListTopicsResponse",
    "MessagePublishRequest",
    "MessagePublishResponse",
    "MessageResponse",
    "OperationMetadata",
    "PartitionAssignment",
    "PartitionAssignmentAck",
    "PartitionAssignmentRequest",
    "PartitionAssignmentServiceAsyncClient",
    "PartitionAssignmentServiceClient",
    "PartitionCursor",
    "PubSubMessage",
    "PublishRequest",
    "PublishResponse",
    "PublisherServiceAsyncClient",
    "PublisherServiceClient",
    "Reservation",
    "SeekSubscriptionRequest",
    "SeekSubscriptionResponse",
    "SeekRequest",
    "SeekResponse",
    "SequencedCommitCursorRequest",
    "SequencedCommitCursorResponse",
    "SequencedMessage",
    "StreamingCommitCursorRequest",
    "StreamingCommitCursorResponse",
    "SubscribeRequest",
    "SubscribeResponse",
    "SubscriberServiceAsyncClient",
    "SubscriberServiceClient",
    "Subscription",
    "TimeTarget",
    "Topic",
    "TopicPartitions",
    "TopicStatsServiceAsyncClient",
    "TopicStatsServiceClient",
    "UpdateSubscriptionRequest",
    "UpdateTopicRequest",
)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the compute future reservations commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.instances import flags as instance_flags
from googlecloudsdk.command_lib.compute.reservations import flags as reservation_flags


def GetNamePrefixFlag():
  """Gets the --name-prefix flag."""
  help_text = """\
  User provided name prefix for system generated reservations when capacity is
  delivered at start time.
  """
  return base.Argument('--name-prefix', help=help_text)


def GetTotalCountFlag(required=True):
  """Gets the --total-count flag."""
  help_text = """\
  The total number of instances for which capacity assurance is requested at a
  future time period.
  """
  return base.Argument(
      '--total-count', required=required, type=int, help=help_text)


def GetStartTimeFlag(required=True):
  """Gets the --start-time flag."""
  return base.Argument(
      '--start-time', required=required, type=str, help=GetStartTimeHelpText())


def GetStartTimeHelpText():
  """Gets the --start-time help text."""
  help_text = """\
  Start time of the Future Reservation. The start time must be an RFC3339 valid
  string formatted by date, time, and timezone or "YYYY-MM-DDTHH:MM:SSZ"; where
  YYYY = year, MM = month, DD = day, HH = hours, MM = minutes, SS = seconds, and
  Z = timezone (i.e. 2021-11-20T07:00:00Z).
  """
  return help_text


def GetEndTimeHelpText():
  """Gets the --end-time help text."""
  help_text = """\
  End time of the Future Reservation. The end time must be an RFC3339 valid
  string formatted by date, time, and timezone or "YYYY-MM-DDTHH:MM:SSZ"; where
  YYYY = year, MM = month, DD = day, HH = hours, MM = minutes, SS = seconds, and
  Z = timezone (i.e. 2021-11-20T07:00:00Z).
  """
  return help_text


def GetDurationHelpText():
  """Gets the --duration help text."""
  help_text = """\
  Alternate way of specifying time in the number of seconds to terminate
  capacity request relative to the start time of a request.
  """
  return help_text


def GetSharedSettingFlag(custom_name=None):
  """Gets the --share-setting flag."""
  help_text = """\
  Specify if this reservation is shared, and if so, the type of sharing. If you
  omit this flag, this value is local (not shared) by default.
  """
  return base.Argument(
      custom_name if custom_name else '--share-setting',
      choices=['local', 'projects'],
      help=help_text)


def GetShareWithFlag(custom_name=None):
  """Gets the --share-with flag."""
  help_text = """\
  If this reservation is shared (--share-setting is not local), provide a list
  of all of the specific projects that this reservation is shared with. List
  must contain project IDs or project numbers.
  """
  return base.Argument(
      custom_name if custom_name else '--share-with',
      type=arg_parsers.ArgList(min_length=1),
      metavar='PROJECT',
      help=help_text)


def AddCreateFlags(
    parser,
    support_location_hint=False,
    support_share_setting=False,
    support_fleet=False,
):
  """Adds all flags needed for the create command."""
  GetNamePrefixFlag().AddToParser(parser)
  GetTotalCountFlag().AddToParser(parser)
  reservation_flags.GetDescriptionFlag().AddToParser(parser)

  group = base.ArgumentGroup(
      'Manage the specific SKU reservation properties.', required=True)
  group.AddArgument(reservation_flags.GetMachineType())
  group.AddArgument(reservation_flags.GetMinCpuPlatform())
  group.AddArgument(reservation_flags.GetLocalSsdFlag())
  group.AddArgument(reservation_flags.GetAcceleratorFlag())
  if support_location_hint:
    group.AddArgument(reservation_flags.GetLocationHint())
  if support_fleet:
    group.AddArgument(instance_flags.AddMaintenanceFreezeDuration())
    group.AddArgument(instance_flags.AddMaintenanceInterval())
  group.AddToParser(parser)

  time_window_group = parser.add_group(
      help='Manage the time specific properties for requesting future capacity',
      required=True)
  time_window_group.add_argument(
      '--start-time', required=True, help=GetStartTimeHelpText())
  end_time_window_group = time_window_group.add_mutually_exclusive_group(
      required=True)
  end_time_window_group.add_argument('--end-time', help=GetEndTimeHelpText())
  end_time_window_group.add_argument(
      '--duration', type=int, help=GetDurationHelpText())

  if support_share_setting:
    share_group = base.ArgumentGroup(
        'Manage the properties of a shared reservation.', required=False)
    share_group.AddArgument(GetSharedSettingFlag())
    share_group.AddArgument(GetShareWithFlag())
    share_group.AddToParser(parser)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unset-value command for the Resource Settings CLI."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.resourcesettings import utils as api_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.resource_settings import arguments
from googlecloudsdk.command_lib.resource_settings import utils


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class UnsetValue(base.DescribeCommand):
  r"""Remove the value of a resource setting.

  Remove the value of a resource setting. This reverts the resource to
  inheriting the resource settings from above it in the resource hierarchy,
  if any is set on a resource above it.

  ## EXAMPLES

  To unset the resource settings ``net-preferredDnsServers'' with the
  project ``foo-project'', run:

    $ {command} net-preferredDnsServers --project=foo-project
  """

  @staticmethod
  def Args(parser):
    arguments.AddSettingsNameArgToParser(parser)
    arguments.AddResourceFlagsToParser(parser)

  def Run(self, args):
    """Unset the resource settings.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the Args method.

    Returns:
       The deleted settings.
    """

    settings_service = api_utils.GetServiceFromArgs(args)
    setting_path = utils.GetSettingsPathFromArgs(args)
    etag = input.etag if hasattr(input, 'etag') else None
    delete_request = api_utils.GetPatchRequestFromArgs(args, setting_path, None,
                                                       etag)
    setting_value = settings_service.Patch(delete_request)

    return setting_value



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Describe worker pool command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.fleet.build import utils
from googlecloudsdk.command_lib.container.fleet.features import base as hubbase
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Describe(hubbase.DescribeCommand):
  """Describe a worker pool used by Google Cloud Build."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
          To get information about a worker pool named `wp1` in region `us-central1`, run:

            $ {command} wp1 --region=us-central1
          """,
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    parser.add_argument(
        '--region',
        required=True,
        help='The Cloud region where the worker pool is.')
    parser.add_argument(
        'WORKER_POOL', help='The ID of the worker pool to describe.')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """

    wp_region = args.region

    release_track = self.ReleaseTrack()
    client = cloudbuild_util.GetClientInstance(release_track)
    messages = cloudbuild_util.GetMessagesModule(release_track)

    parent = properties.VALUES.core.project.Get(required=True)

    wp_name = args.WORKER_POOL

    # Get the workerpool ref
    wp_resource = resources.REGISTRY.Parse(
        None,
        collection='cloudbuild.projects.locations.workerPools',
        api_version=cloudbuild_util.RELEASE_TRACK_TO_API_VERSION[release_track],
        params={
            'projectsId': parent,
            'locationsId': wp_region,
            'workerPoolsId': wp_name,
        })

    # Send the Get request
    wp = client.projects_locations_workerPools.Get(
        messages.CloudbuildProjectsLocationsWorkerPoolsGetRequest(
            name=wp_resource.RelativeName()))

    if release_track != base.ReleaseTrack.ALPHA:
      if wp.hybridPoolConfig is not None:
        raise exceptions.Error('NOT_FOUND: Requested entity was not found.')

    # Format the workerpool name for display
    try:
      wp.name = cloudbuild_util.WorkerPoolShortName(wp.name)
    except ValueError:
      pass  # Must be an old version.

    return wp


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class DescribeBeta(Describe):
  """Describe a worker pool used by Google Cloud Build."""


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class DescribeAlpha(Describe):
  """Describe a private or hybrid worker pool used by Google Cloud Build."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
            To get information about a private or hybrid worker pool named `wp1` in region `us-central1`, run:

              $ {command} wp1 --region=us-central1
            """,
  }

  feature_name = 'cloudbuild'

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    parser.add_argument(
        '--region',
        required=True,
        help='The Cloud region where the worker pool is.')
    parser.add_argument(
        'WORKER_POOL', help='The ID of the worker pool to describe.')
    parser.display_info.AddFormat("""
      multi(wp_status:format='table[box](
          NAME:label=NAME:sort=1,
          TYPE:label="TYPE",
          HWP_DESCRIPTION:label="CLUSTER DESCRIPTION":optional,
          HWP_STATUS:label=STATUS:optional
      )',
          wp_config:format=default)
    """)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """

    wp = super(DescribeAlpha, self).Run(args)

    wp_status = {
        'NAME': wp.name,
    }
    wp_out = {
        'wp_config': wp,
        'wp_status': wp_status
    }

    if wp.privatePoolV1Config is not None:
      wp_status[
          'TYPE'] = cloudbuild_util.WorkerpoolTypes.PRIVATE.name.capitalize()
    elif wp.hybridPoolConfig is not None:
      feature = self.GetFeature(v1alpha1=True)
      feature_state_memberships = utils.GetFeatureStateMemberships(feature)
      details = feature_state_memberships[wp.hybridPoolConfig.membership]
      wp_status.update({
          'TYPE': cloudbuild_util.WorkerpoolTypes.HYBRID.name.capitalize(),
          'HWP_DESCRIPTION': details.description,
          'HWP_STATUS': details.code,
      })
    else:
      wp_status[
          'TYPE'] = cloudbuild_util.WorkerpoolTypes.UNKNOWN.name.capitalize()

    return wp_out
